#! /bin/sh
#
# -----------------------------------
# Initscript for the ...
# -----------------------------------
# 
#

set -e

PATH=/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin
DESC="VKC4 ConverterService"
NAME=ConverterService
CONVERTERDIR=/usr/share/VKC4/converter

cd $CONVERTERDIR
lib=
for file in lib/*.jar
do
	lib=$file:$lib
done

MAIN="vkc.converter.server.ConverterServer"

DAEMON="/usr/bin/java -server -Xmx128m -XX:PermSize=64m -Dfile.encoding=ISO-8859-1 -Djava.awt.headless=true -classpath .:${lib} $MAIN"
PIDFILE=/var/run/$NAME.pid
SCRIPTNAME=/etc/init.d/$NAME.sh

# Gracefully exit if the package has been removed.
test -x /usr/bin/java || exit 0

# ---------------------------------------
# Function that starts the daemon/service
# ---------------------------------------
d_start()
{
$DAEMON &> /dev/null & echo $! > $PIDFILE
}

# --------------------------------------
# Function that stops the daemon/service
# --------------------------------------
d_stop()
{
start-stop-daemon --stop --quiet --pidfile $PIDFILE
}

case "$1" in
start)
echo -n "Starting $DESC: $NAME"
d_start
echo "."
;;
stop)
echo -n "Stopping $DESC: $NAME"
d_stop
echo "."
;;
restart|force-reload)
echo -n "Restarting $DESC: $NAME"
d_stop
sleep 1
d_start
echo "."
;;
*)
echo "Usage: $SCRIPTNAME {start|stop|restart|force-reload}" >&2
exit 1
;;
esac

exit 0