/*
 * $Id: MsPowerpointDocConverter.java,v 1.18 2007/07/10 11:24:03 marbobo Exp $
 *
 * Copyright (c) Siemens AG 2008 
 *           (c) Universitt Paderborn 2008 
 * Licensed under EUPL 1.1. 
 *
 * $Log: MsPowerpointDocConverter.java,v $
 * Revision 1.18  2007/07/10 11:24:03  marbobo
 * + Bugfix: Dateiname die an ProcessKiller bergeben wird korrigiert
 *
 * Revision 1.17  2006/06/06 20:36:01  marbobo
 * + Mgliche NPE abgefangen
 *
 * Revision 1.16  2006/04/18 09:23:54  korowa
 * Nur erlaubte Konvertierungen werden durchgefhrt(DocConverter:conversions) bzw. zurckgeliefert.
 *
 * Revision 1.15  2006/03/23 11:36:42  marbobo
 * + Mgliche NPE abgefangen
 *
 * Revision 1.14  2005/04/18 12:22:48  lawern
 * Timeout in wscript-Aufruf eingebaut.
 *
 * Revision 1.13  2004/11/10 15:02:44  koppy
 * Projektweite berarbeitung von Error-Logs, Stacktraces,..
 *
 * Revision 1.12  2004/08/31 14:15:49  lawern
 * ProcessKiller eingebaut
 *
 * Revision 1.11  2004/06/27 17:12:46  marbobo
 * + Quellcode maintenance
 *
 * Revision 1.10  2004/03/02 14:26:11  lawern
 * Anpassung an OfficeXP.
 *
 * Revision 1.9  2003/11/29 22:13:53  koppy
 * InputStreams in VKCInputStream gekappselt
 *
 * Revision 1.8  2003/11/11 08:52:27  lawern
 * Fehler behoben.
 *
 * Revision 1.7  2003/09/23 11:34:31  lawern
 * auf absolute Pfade umgestellt
 *
 * Revision 1.6  2003/09/11 08:27:23  lawern
 * einige Methoden public gemacht
 *
 * Revision 1.5  2003/07/28 11:55:30  lawern
 * *** empty log message ***
 *
 * Revision 1.4  2003/07/08 16:38:13  lawern
 * Anpassung an Office XP
 *
 * Revision 1.3  2003/06/26 15:15:22  lawern
 * XLS und PPT eingefgt und Konverter berarbeitet
 *
 */
package vkc.converter;

import vkc.misc.ProcessKiller;
import vkc.misc.VKCLogManager;
import vkc.misc.VKCProcess;

import vkc.util.FileUtils;
import vkc.util.VKCInputStream;
import vkc.util.ZipUtils;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import java.net.URL;

import java.util.ArrayList;
import java.util.Collection;

/**
 * Klasse zum Konvertieren von Textdokumenten.
 *
 * @author lawern
 */
public class MsPowerpointDocConverter extends DocConverter {
	private static final boolean debug = false;

	public static String docSuffix = ".ppt";

	private static Object semaphor = new Object();

	private String[] cmdarr = new String[6];

	private String[] env = null;

	private File dir = null;

	private File zipFile = null;

	private String htmlFile = null;

	private File htmlDir = null;

	private HtmlDocConverter htmlConverter = null;

	public MsPowerpointDocConverter() {
	}

	/**
	 * Liefert den Suffix fr temporre Dateien zurck.
	 *
	 * @return docSuffix
	 */
	public String getDocSuffix() {
		return docSuffix;
	}

	/**
	 * Diese Methode konvertiert das bergebene Dokument in die in der
	 * Collection definierten Formate. Der Parameter conversions wird zur Zeit
	 * noch nicht ausgewertet.
	 *
	 * @param docFile URL des zu konvertierendes Dokument
	 * @param conversions Collection von VKCDocumentTypes welche die zu
	 *        erzeugenden Konvertierungen beschreibt.
	 */
	//PDF-Konvertierung wird von MS-Office noch nicht untersttzt-->spter implementieren
	public void convert(URL docFile, Collection conversions) {
		if (debug) {
			System.out.println("MsPowerpointDocConverter: " + docFile);
		}

		InputStream docStream = null;
		File tmpFile = null;

		try {
			//Datei von Url in temporre Datei schreiben
			docStream = docFile.openStream();
			tmpFile = File.createTempFile("PPTDoc", ".ppt");

			if (!debug) {
				tmpFile.deleteOnExit();
			}

			FileUtils.saveInputStream(tmpFile, docStream, true);
			FileUtils.closeInputStream(docStream);

			//String docFileString = VKCUtils.getLocalPath(docFile);
			VKCProcess process = null;

			dir = new File(basePath).getAbsoluteFile();
			
			if (enableHtml || enableTdf) {
				try {
					htmlDir = File.createTempFile("PPTDoc", "");
					htmlDir.delete();
					htmlDir.mkdir();

					if (!debug) {
						htmlDir.deleteOnExit();
					}

					htmlFile = htmlDir.getCanonicalPath() + File.separator + "index.html";
				} catch (java.io.IOException ioe) {
					VKCLogManager.errorLog(this, "3", "Can't create temp-file.", ioe);
				}

				// Kommando-Array mit Voreinstellung fr den Konverter
				// initialisieren
				cmdarr[0] = new File("wscript").toString();
				cmdarr[1] = "PPTToHTML.vbs";
				cmdarr[2] = "//B";
				cmdarr[3] = "//T:" + (timeout * 60 - 5);
				cmdarr[4] = tmpFile.getCanonicalPath();
				cmdarr[5] = htmlFile;

				//immer nur ein PPT starten
				synchronized (semaphor) {
					String tempFile = tmpFile.getCanonicalPath();
					tempFile = tempFile.substring(tempFile.lastIndexOf(File.separator) + 1);

					ProcessKiller killer = new ProcessKiller(tempFile, timeout + 8);
					killer.start();
					process = new VKCProcess(cmdarr, env, dir);
					process.executeAndWaitWithTimeout(timeout * 60);
					killer.interrupt();
				}

				FileUtils.deleteZeroByteFile(new File(htmlFile));
			}

			if (htmlFile != null && !new File(htmlFile).exists()) {
				VKCLogManager.errorLog(this, "6", " HTML conversion not generated.");

				try {
					FileUtils.deleteDirectory(htmlDir);
				} catch (Exception ex) {
					VKCLogManager.errorLog(this, "", ex.getMessage(), ex);
				}
			} else {
				if (enableHtml) {
					try {
						//Html-Version zusammenzipen
						zipFile = File.createTempFile("PPTDoc", ".zip");

						if (!debug) {
							zipFile.deleteOnExit();
						}

						ZipUtils.zip(htmlDir, zipFile, true, htmlDir.getCanonicalPath() + File.separator);
					} catch (java.io.IOException ioe) {
						VKCLogManager.errorLog(this, "4", "Can't create temp-file.", ioe);
					}
				}

				//TDL-Stream erzeugen
				//Der bergeben null-Parameter muss spter durch eine Collection ersetzt werden,
				//welche dafr sorgt, dass nur ein Tdf-Stream erzeugt wird!!!
				if (enableTdf) {
					File[] files = htmlDir.listFiles();
					File subDir = null;

					//Das von PPT erstellte Unterverzeichnis suchen
					for (int i = files.length - 1; i >= 0; i--) {
						if (files[i].isDirectory()) {
							subDir = files[i];
						}
					}

					//alle .htm-Dateien im Unterverzeichnis konvertieren
					if (subDir != null) {
						File[] htm_files = FileUtils.listFilesBySuffix(subDir, ".htm");
						File[] html_files = FileUtils.listFilesBySuffix(subDir, ".html");
						files = new File[htm_files.length + html_files.length];

						System.arraycopy(htm_files, 0, files, 0, htm_files.length);
						System.arraycopy(html_files, 0, files, htm_files.length, html_files.length);

						if (files.length > 0) {
							Collection urls = new ArrayList(files.length);

							for (int i = 0; i < files.length; i++) {
								urls.add(files[i].toURL());
							}

							htmlConverter = new HtmlDocConverter();
							htmlConverter.convert(urls, null);
						} else {
							//Falls im Unterverzeichnis keine HTML-Dateien existieren
							//nur die Hauptdatei konvertieren
							htmlConverter = new HtmlDocConverter();
							htmlConverter.convert(new File(htmlFile).toURL(), null);
						}
					} else {
						//Falls kein Unterverzeichnis existiert nur die HTML-Datei
						//konvertieren
						htmlConverter = new HtmlDocConverter();
						htmlConverter.convert(new File(htmlFile).toURL(), null);
					}
				}
				//Html-Verzeichnis lschen
				try {
					FileUtils.deleteDirectory(htmlDir);
				} catch (Exception ex) {
					VKCLogManager.errorLog(this, "5", "Can't delete temp-dir.", ex);
				}
			}
		} catch (IOException ioe) {
			if (tmpFile != null) {
				VKCLogManager.errorLog(this, "8", "Can't create temp-file: " + tmpFile.getPath(), ioe);
			} else {
				VKCLogManager.errorLog(this, "8", "Can't create temp-file: ", ioe);
			}

			FileUtils.closeInputStream(docStream);
		} finally {
			//temporre Dokumentendatei lschen
			if (!debug) {
				if (tmpFile != null) {
					tmpFile.delete();
				}
			}
		}
	}

	/**
	 * Liefert nach dem Aufruf der convert-Methode den Stream der
	 * Tdf-Konvertierung.
	 *
	 * @return Stream der Tdf-Konvertierung oder <code>null</code> falls er
	 *         nicht existiert
	 */
	public VKCInputStream getTdfStream() {
		if (htmlConverter != null && enableTdf) {
			return htmlConverter.getTdfStream();
		}

		return null;
	}

	/**
	 * Liefert nach dem Aufruf der convert-Methode den Stream der
	 * HTML-Konvertierung.
	 *
	 * @return Stream der HTML-Konvertierung oder <code>null</code> falls er
	 *         nicht existiert
	 */
	public VKCInputStream getHtmlStream() {
		if (zipFile != null && enableHtml) {
			incOpenStreams();

			try {
				return new VKCInputStream(new BufferedInputStream(new FileInputStream(zipFile)), zipFile.length()) {
					public void close() throws IOException {
						super.close();
						closeHtmlStream();
					}
				};
			} catch (Exception e) {
				VKCLogManager.errorLog(this, "2", "ZIP-File not found!", e);
			}
		}

		return null;
	}

	/**
	 * Gibt eine Liste zurueck, der alle HREFs einer Seite enthlt.
	 *
	 * @return Link-Vektor
	 */
	public ArrayList getLinks() {
		if (htmlConverter != null) {
			return htmlConverter.getLinks();
		}

		return null;
	}

	/**
	 * Liefert den Titel des Dokuments zurck.
	 *
	 * @return Titel des Dokuments oder <code>null</code>
	 */
	public String getTitle() {
		if (htmlConverter != null) {
			return htmlConverter.getTitle();
		}

		return null;
	}

	/**
	 * Liefert eine Collection mit Objekten vom Typ TdfElement zurck, welche
	 * das Dokument beschreiben.
	 *
	 * @return Collection mit Objekten vom Typ TdfElement
	 */
	public Collection getElements() {
		if (htmlConverter != null) {
			return htmlConverter.getElements();
		}

		return null;
	}

	/**
	 * Lschen der TDF-Streams
	 */
	void closeTdfStream() {
		if (debug) {
			System.out.println("closeTdfStream");
		}

		decOpenStreams();
	}

	/**
	 * Lschen des Ascii-Streams
	 */
	void closeHtmlStream() {
		if (debug) {
			System.out.println("closeHtmlStream");
		}

		decOpenStreams();

		//ZIP-Datei lschen
		if (zipFile != null) {
			zipFile.delete();
		}
	}

	/**
	 * Schliet das Konverter-Objekt und fgt es wieder in den Konverter-Pool
	 * ein.
	 */
	final void closeConverter() {
		//temporre Dateien lschen
		if (zipFile != null) {
			zipFile.delete();
		}
		//this-Referenz wieder in die Queue einfgen
	}
}