/*
 * $Id: RestoreAccountAction.java,v 1.16 2007/08/08 12:14:52 marbobo Exp $
 *
 * Copyright (c) Siemens AG 2008 
 *           (c) Universitt Paderborn 2008 
 * Licensed under EUPL 1.1. 
 *
 * $Log: RestoreAccountAction.java,v $
 * Revision 1.16  2007/08/08 12:14:52  marbobo
 * + License-Funktionalitt komplett umgeschrieben
 * + Bugfix bzgl. Beschreibungsfelder von Entitten
 * + Bugfix mailing
 *
 * Revision 1.15  2006/10/04 03:14:32  marbobo
 * + Vollstndige Untersttzung fr Templates implementiert
 * + Alles von Virtoweb auf ProCoBa umgestellt
 *
 * Revision 1.14  2006/04/29 18:39:46  marbobo
 * + Refactoriengs und eine Ausgaben vereinheitlicht
 *
 * Revision 1.13  2006/02/27 16:16:00  marbobo
 * + Systemsichten hinzugefgt
 * + SysLinks angepasst
 * + Benutzer knnen falls gewnscht Dokumentqueries anlegen
 *
 * Revision 1.12  2005/12/16 11:50:44  lawern
 * *** empty log message ***
 *
 * Revision 1.11  2005/10/03 17:34:21  marbobo
 * + Nur zur Projektsicht leiten wenn Projektmanagment aktiv ist
 *
 * Revision 1.10  2005/09/01 08:31:59  korowa
 * Es werden mehrere URL angeboten, welche direkten Zugriff auf Dateien
 *  und automatisches Anmelden als Gast erlauben.
 *
 * Revision 1.9  2005/03/16 15:48:47  jonyp
 * getLastnameLabel for user tracking infos
 *
 * Revision 1.8  2005/03/16 15:10:13  jonyp
 * *** empty log message ***
 *
 * Revision 1.7  2005/02/14 14:01:51  bigmac
 * Applikationsweites CSS konfigurierbar gemacht + Refactorings
 *
 * Revision 1.6  2005/02/14 01:29:52  marbobo
 * + Ungltige Session-Attribute entfernen
 *
 * Revision 1.5  2004/11/02 21:42:30  marbobo
 * + Einige Funktionen fr Gast-Benutzer gesperrt
 * + WebShare.LOGIN_PAGE mit WebShare.getLoginPage() ersetzt
 *
 * Revision 1.4  2004/06/27 17:36:01  marbobo
 * + Quellcode maintenance
 *
 * Revision 1.3  2004/04/01 20:50:14  marbobo
 * + getVirtowebSysLink()
 *
 * Revision 1.2  2004/04/01 13:33:19  jonyp
 * deleted unused imports
 *
 * Revision 1.1  2004/04/01 09:39:38  jonyp
 * first strike
 *
 */
package vkc.view.action;

import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import vkc.db.entities.VKCUser;
import vkc.db.entities.VKCCategory;
import vkc.db.serializer.VKCSysParameterSerializer;

import vkc.misc.VKCLogManager;

import vkc.util.ServletUtils;
import vkc.view.PageId;
import vkc.view.WebShare;
import vkc.view.bean.UserTrackingInfo;
import vkc.view.form.SysLinkForm;

import java.util.Date;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

/**
 * class for the control of restoring a previous account
 *
 * @author jonyP
 */
public final class RestoreAccountAction extends Action {
	private ActionForward INFO_FORWARD = new ActionForward("/templates/${template.name}/common/InfoPage.jsp");
    
    public ActionForward execute(ActionMapping mapping, 
            					 ActionForm form,
            					 HttpServletRequest request, 
            					 HttpServletResponse response)
    throws Exception {
        HttpSession session = request.getSession();

        // get the user, who has captured an account
        VKCUser user = (VKCUser) session.getAttribute(WebShare.PREV_USER_KEY);
        
        // check, if the capturer has enough privileges...that means: must be an admin
        // or is not in the session
		if (user == null || !user.hasAdminPrivileges()) {
			request.setAttribute(WebShare.INFO_KEY, "vkc.info.action_not_allowed");
			
			return INFO_FORWARD;
		}
        
        // drop capturer from session
        ServletUtils.removeSessionAttributes(session);
        
        // save captureUser as new User in session
		if (user.getSessionTimeout() != null && user.getSessionTimeout().intValue() != 0) {
			// Session-Timeout setzen
			session.setMaxInactiveInterval(user.getSessionTimeout().intValue() * 60);
		}
        
        // save previousUser in session
        session.setAttribute(WebShare.USER_KEY, user);
        session.setAttribute(WebShare.LOCALE_KEY, user.getLanguage().getLocale());

        // change language
        setLocale(request, user.getLanguage().getLocale());

        String log = "User '" + user.getUser() + "' has restored his account";
		log +=  " (userId = " +  user.getUserId() + ", session timeout = " + (session.getMaxInactiveInterval() / 60) + "min)";

        VKCLogManager.infoLog(this, log);

        UserTrackingInfo bean = new UserTrackingInfo(user.getUserId(),
                                                     user.getLastnameLabel(),
                									 request.getSession(), 
                									 request.getRemoteAddr(), 
                									 new Date(user.getMostRecentLogin().getTime()));

        session.setAttribute(WebShare.USER_TRACKING_KEY, bean);
        session.setAttribute(WebShare.SYSTEMVIEWTYPEID_KEY, user.getSystemViewTypeId());

        //Procoba-Start
        if (PageId.FUNCTIONAL_VIEW.equals(user.getSystemViewTypeId()) && VKCSysParameterSerializer.isProjectManagementEnabled()) {
			return new ActionForward(SysLinkForm.getProcobaFunctionalSysLink());
        }

		if (PageId.PROJECT_VIEW.equals(user.getSystemViewTypeId()) && VKCSysParameterSerializer.isProjectManagementEnabled()) {
			return new ActionForward(SysLinkForm.getProcobaProjectSysLink());
		} 

		// by default: startCategory is root
		Integer startCategoryId = new Integer(VKCCategory.ROOT_CATEGORY_ID); 
	    
		// if user has another take it...
		if (user.getHomeCategoryId() != null && user.getHomeCategoryId().intValue() > VKCCategory.ROOT_CATEGORY_ID) {
			startCategoryId = user.getHomeCategoryId();
		}

		return new ActionForward(SysLinkForm.getCategorySysLink(startCategoryId, SysLinkForm.LOGINTYPE_NORMAL_USER), true);
    }
}